import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import controlP5.*; 
import processing.serial.*; 
import msafluid.*; 
import javax.media.opengl.GL2; 
import processing.opengl.*; 
import javax.media.opengl.*; 
import java.nio.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class BotbaitFishing extends PApplet {

/*
 * Botbait Fishing!
 * ::.:.:..::.::.::.
 *
 * Visualization of pixelated fish swimming around in
 * cosmic-coloured fluid. Botbait the tentacle mechanism
 * robot can disturb the fish in different ways, triggered
 * by your interaction with it.
 *
 * For more info:
 * http://robobrrd.com/tentacle
 * 
 * By RobotGrrl for RoboBrrd.com
 * Nov. 7, 2013
 *
 * CC BY-SA
 */








static final boolean DEBUG = false;

final float FLUID_WIDTH = 120;
static final int numFish = 5;

MSAFluidSolver2D fluidSolver;
ParticleSystem particleSystem;

float invWidth, invHeight;
float aspectRatio, aspectRatio2;
PImage imgFluid;

PImage fishImg[] = new PImage[numFish];

boolean drawFluid = true;
// ---


public void setup() {
  //size(960, 640, P3D);
  size(displayWidth * 49/50, displayHeight * 49/50, P3D);
  noStroke();

  invWidth = 1.0f/width;
  invHeight = 1.0f/height;
  aspectRatio = width * invHeight;
  aspectRatio2 = aspectRatio * aspectRatio;

  // create fluid and set options
  fluidSolver = new MSAFluidSolver2D((int)(FLUID_WIDTH), (int)(FLUID_WIDTH * height/width));
  //fluidSolver.enableRGB(true).setFadeSpeed(0.00003).setDeltaT(0.7).setVisc(0.0001);
  fluidSolver.enableRGB(true).setFadeSpeed(0.003f).setDeltaT(0.7f).setVisc(0.0001f);

  // create image to hold fluid picture
  imgFluid = createImage(fluidSolver.getWidth(), fluidSolver.getHeight(), RGB);

  // create particle system
  particleSystem = new ParticleSystem();

  // fish images
  for(int i=0; i<numFish; i++) {
    fishImg[i] = loadImage("fish.png");
  }

  initControlP5();

}


public void draw() {

  doInitOcean();
  
  doBlurps();
  doCornerBlurps();
  doSideBlurps();
  
  fluidSolver.update();

  if(drawFluid) {
      for(int i=0; i<fluidSolver.getNumCells(); i++) {
          int d = 2;
          imgFluid.pixels[i] = color(fluidSolver.r[i] * d, fluidSolver.g[i] * d, fluidSolver.b[i] * d);
      }  
      imgFluid.updatePixels();//  fastblur(imgFluid, 2);
      imageMode(CORNER);
      image(imgFluid, 0, 0, width, height);
  } 

  particleSystem.updateAndDraw();
  
  rotateFishes();
  
  if(guiVisible) {
    cp5.draw();
    cp5_v2.draw();
  }
  
  checkForFishies();
  
  readData();
  
}


public void mousePressed() {

}


public void mouseMoved() {
  float mouseNormX = mouseX * invWidth;
  float mouseNormY = mouseY * invHeight;
  float mouseVelX = (mouseX - pmouseX) * invWidth;
  float mouseVelY = (mouseY - pmouseY) * invHeight;

  addForce(mouseNormX, mouseNormY, mouseVelX, mouseVelY, generateColour(mouseNormX, mouseNormY));
}


public void keyPressed() {
    switch(key) {
    case 'r': 
        renderUsingVA ^= true; 
        println("renderUsingVA: " + renderUsingVA);
        break;
    case 'h':
      guiVisible = !guiVisible;
      break;
    }
    println(frameRate);
}


// add force and dye to fluid, and create particles
public void addForce(float x, float y, float dx, float dy, int c) {
  float speed = dx * dx  + dy * dy * aspectRatio2;    // balance the x and y components of speed with the screen aspect ratio

  if(speed > 0) {
    if(x<0) x = 0; 
    else if(x>1) x = 1;
    if(y<0) y = 0; 
    else if(y>1) y = 1;

    float colorMult = 5;
    float velocityMult = 30.0f;

    int index = fluidSolver.getIndexForNormalizedPosition(x, y);

    int drawColor;

    drawColor = c;
    
    colorMode(RGB, 1);

    fluidSolver.rOld[index]  += red(drawColor) * colorMult;
    fluidSolver.gOld[index]  += green(drawColor) * colorMult;
    fluidSolver.bOld[index]  += blue(drawColor) * colorMult;

    particleSystem.addParticles(x * width, y * height, 10);
    fluidSolver.uOld[index] += dx * velocityMult;
    fluidSolver.vOld[index] += dy * velocityMult;
  }
}


public int generateColour(float x, float y) {
  colorMode(HSB, 360, 1, 1);
  float hue = ((x + y) * 180 + frameCount) % 360;
  int drawColor = color(hue, 1, 1);
  return drawColor;
}


// -- ARDUINO -- //
Serial arduino;
int port = 99;
boolean connected = false;

// -- PARSING -- //
int msgLen = 12;
int msgIndex = 0;
char[] msg = new char[msgLen];
boolean reading = false;
boolean completed = false;
int receivedValue = 0;

// -- SEND -- //
static String moveXSerial = "MX";
static String moveYSerial = "MY";
static String fishNumSerial = "FF";

// -- RECEIVE -- //
static String disturbSerial = "DS";
static String inkSerial = "IN";


public void readData() {
 
  if(connected) {
  
    // let's read the data
    
    char m;
    
    while(arduino.available() > 0) {
      
      m = arduino.readChar();
     
      if(m == '!') {
        if(DEBUG) println("\nping");
        reading = true;
      }
      
      if(DEBUG) print(m + " ");
      
      if(reading) {
        if(msgIndex < msgLen) msg[msgIndex++] = m;
      }
      
      if(m == '~') {
        if(DEBUG) println("\nding");
        reading = false;
        completed = true;
      }
      
    }
    
    if(completed) {
      if(DEBUG) println("\ngood job");
      
      for(int i=0; i<msgIndex; i++) {
        if(DEBUG) print(msg[i] + " "); 
      }
      if(DEBUG) print("\n");
      
      String cmd = "";
      if(msg[0] == '!') {
        cmd += ("" + msg[1] + "" + msg[2]);
      }
      if(DEBUG) println("cmd: " + cmd);
      
      String val = "";
      if(msg[3] == ':') {
        
        for(int i=4; i<(msgIndex-1); i++) {
          val += msg[i];
        }
        
      }
      if(DEBUG) println("val: " + val);
      
      boolean good = false;
      if(msg[(msgIndex-1)] == '~') {
        good = true;
      }
      
      if(good) {
        if(DEBUG) println("good!");
          parseCommand(cmd, val);
      }
      
      msgIndex = 0;
      completed = false;
      
    }
    
  }
  
}


public void parseCommand(String cmd, String val) {
 
 if(cmd.equals(disturbSerial)) {
    if(val != "" && val.length() < 5) receivedValue = (int)Integer.parseInt(val);
    for(int ii=0; ii<receivedValue; ii++) {
      disturbance();
    }
  } else if(cmd.equals(inkSerial)) {
    inkance();
  }
  
}


public void sendSerial(String sl, int val) {
  arduino.write("!" + sl + ":" + val + "~");
  if(DEBUG) print(">");
}

/***********************************************************************
 
 Copyright (c) 2008, 2009, Memo Akten, www.memo.tv
 *** The Mega Super Awesome Visuals Company ***
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of MSA Visuals nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE. 
 *
 * ***********************************************************************/ 

class Particle {
    final static float MOMENTUM = 0.7f;
    final static float FLUID_FORCE = 0.9f;

    float x, y;
    float x0, y0;
    float vx, vy;
    float radius;       // particle's size
    float alpha;
    float mass;

    public void init(float x, float y) {
        this.x = x;
        this.y = y;
        vx = 0;
        vy = 0;
        radius = 5;
        alpha  = random(0.3f, 1);
        mass = random(0.1f, 1);
    }


    public void update() {
        // only update if particle is visible
        if(alpha == 0) return;
        
        x0 = x;
        y0 = y;

        // read fluid info and add to velocity
        int fluidIndex = fluidSolver.getIndexForNormalizedPosition(x * invWidth, y * invHeight);
        vx = fluidSolver.u[fluidIndex] * width * mass * FLUID_FORCE + vx * MOMENTUM;
        vy = fluidSolver.v[fluidIndex] * height * mass * FLUID_FORCE + vy * MOMENTUM;

        // update position
        x += vx;
        y += vy;

        // bounce of edges
        if(x<0) {
            x = 0;
            vx *= -1;
        }
        else if(x > width) {
            x = width;
            vx *= -1;
        }

        if(y<0) {
            y = 0;
            vy *= -1;
        }
        else if(y > height) {
            y = height;
            vy *= -1;
        }

        // hackish way to make particles glitter when the slow down a lot
        if(vx * vx + vy * vy < 1) {
            vx = random(-1, 1);
            vy = random(-1, 1);
        }

        // fade out a bit (and kill if alpha == 0);
        alpha *= 0.999f;
        if(alpha < 0.01f) alpha = 0;

    }


    public void updateVertexArrays(int i, FloatBuffer posBuffer, FloatBuffer colBuffer) {
        int vi = i * 4;
        posBuffer.put(vi++, x - vx);
        posBuffer.put(vi++, y - vy);
        posBuffer.put(vi++, x);
        posBuffer.put(vi++, y);

        int ci = i * 6;
        colBuffer.put(ci++, alpha);
        colBuffer.put(ci++, alpha);
        colBuffer.put(ci++, alpha);
        colBuffer.put(ci++, alpha);
        colBuffer.put(ci++, alpha);
        colBuffer.put(ci++, alpha);
    }


    public void drawOldSchool(GL2 gl) {
        gl.glColor3f(alpha, alpha, alpha);
        gl.glVertex2f(x-vx, y-vy);
        gl.glVertex2f(x, y);
    }

}








/***********************************************************************
 
 Copyright (c) 2008, 2009, Memo Akten, www.memo.tv
 *** The Mega Super Awesome Visuals Company ***
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of MSA Visuals nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE. 
 *
 * ***********************************************************************/ 



boolean renderUsingVA = false; // render using VA doesn't work (for me)

public void fadeToColor(GL2 gl, float r, float g, float b, float speed) {
    gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
    gl.glColor4f(r, g, b, speed);
    gl.glBegin(gl.GL_QUADS);
    gl.glVertex2f(0, 0);
    gl.glVertex2f(width, 0);
    gl.glVertex2f(width, height);
    gl.glVertex2f(0, height);
    gl.glEnd();
}


class ParticleSystem {
    FloatBuffer posArray;
    FloatBuffer colArray;

    final static int maxParticles = 5000;
    int curIndex;

    Particle[] particles;
    Particle[] fishParticles;

    ParticleSystem() {
        particles = new Particle[maxParticles];
        fishParticles = new Particle[numFish];
        for(int i=0; i<maxParticles; i++) particles[i] = new Particle();
        for(int i=0; i<numFish; i++) {
          fishParticles[i] = new Particle();
          fishParticles[i].init(random(0, width), random(0, height));
          fishParticles[i].mass = 0.8f;
        }
        curIndex = 0;
        // TODO: Fix floatbuffer
        posArray = ByteBuffer.allocateDirect( (maxParticles+numFish) * 2 * 2 * 8).asFloatBuffer();// 2 coordinates per point, 2 points per particle (current and previous)
        colArray = ByteBuffer.allocateDirect( (maxParticles+numFish) * 3 * 2 * 8).asFloatBuffer();
    }


    public void updateAndDraw(){
        //OPENGL Processing 2.0
        GL2 gl;
        PGraphicsOpenGL pg = (PGraphicsOpenGL) g;         // processings opengl graphics object
        PGL pgl = pg.beginPGL();                // JOGL's GL object
        gl = pgl.gl.getGL().getGL2();
        
        gl.glEnable( GL.GL_BLEND );             // enable blending
        if(!drawFluid) fadeToColor(gl, 0, 0, 0, 0.05f);

        gl.glBlendFunc(gl.GL_ONE, GL.GL_ONE);  // additive blending (ignore alpha)
        gl.glEnable(gl.GL_LINE_SMOOTH);        // make points round
        gl.glLineWidth(1);


        if(renderUsingVA) {
            for(int i=0; i<maxParticles; i++) {
                if(particles[i].alpha > 0) {
                    particles[i].update();
                    particles[i].updateVertexArrays(i, posArray, colArray);
                }
            }    
            gl.glEnableClientState(gl.GL_VERTEX_ARRAY);
            gl.glVertexPointer(2, gl.GL_FLOAT, 0, posArray);

            gl.glEnableClientState(gl.GL_COLOR_ARRAY);
            gl.glColorPointer(3, gl.GL_FLOAT, 0, colArray);

            gl.glDrawArrays(gl.GL_LINES, 0, maxParticles * 2);
        } 
        else {
            gl.glBegin(gl.GL_LINES);               // start drawing points
            for(int i=0; i<maxParticles; i++) {
                if(particles[i].alpha > 0) {
                    particles[i].update();
                    particles[i].drawOldSchool(gl);    // use oldschool renderng
                }
            }
            for(int i=0; i<numFish; i++) {
              fishParticles[i].update();
              fishParticles[i].drawOldSchool(gl);
              fishParticles[i].alpha = 1.0f;
            }
            gl.glEnd();
        }

        gl.glDisable(GL.GL_BLEND);
        pg.endPGL();
    }


    public void addParticles(float x, float y, int count ){
        for(int i=0; i<count; i++) addParticle(x + random(-15, 15), y + random(-15, 15));
    }


    public void addParticle(float x, float y) {
        particles[curIndex].init(x, y);
        curIndex++;
        if(curIndex >= maxParticles) curIndex = 0;
    }

}



// -- WATER -- //
boolean initedOcean = false;
int initedCnt = 0;
int lastInitOc = 0;
int lastBlurp = 0;
int lastCornerBlurp = 0;
int cornerBlurp = 0;
int lastSideBlurp = 0;
int sideBlurp = 0;
float fishAngle[] = new float[numFish];
boolean disturbanceDir = false;
int lastFishCountSend = 0;


public void initOcean() {
  float vel = 0.3f;
  colorMode(HSB, 360, 1, 1);
  //color c = color(random(185, 250), 1.0, 1.0);
  int c = generateColour(0, 1);
  for(float i=0.1f; i<1.0f; i+=0.1f) {
    c = generateColour(i, 0.0f);
    addForce(i, 0.0f, 0.0f, vel, c);
    
    c = generateColour(i, 1.0f);
    addForce(i, 1.0f, 0.0f, -vel, c);
    
    c = generateColour(0.0f, i);
    addForce(0.0f, i, vel, 0, c);
    
    c = generateColour(1.0f, i);
    addForce(1.0f, i, -vel, 0, c);
  }
}


public void doInitOcean() {
 if(!initedOcean) {
    if(millis()-lastInitOc >= 10) {
      
      initOcean();
      initedCnt++;
      
      if(initedCnt >= 10) {
        initedOcean = true;
      }
      
      lastInitOc = millis();
      
    }
  } 
}


public void doBlurps() {
 if(millis()-lastBlurp >= 500) {
    colorMode(HSB, 360, 1, 1);
    //color c = color(300, 1.0, 1.0);
    int c = generateColour(random(0,1.0f), random(0,1.0f));
    addForce(random(0,1.0f), random(0,1.0f), random(-0.5f,0.5f), random(-0.5f,0.5f), c);
    lastBlurp = millis();
  } 
}


public void doCornerBlurps() {
 if(millis()-lastCornerBlurp >= 1000) {
   
   float vel = 0.2f;
   
    colorMode(HSB, 360, 1, 1);
    //color c = color(300, 1.0, 1.0);
    int c = generateColour(random(0,1.0f), random(0,1.0f));
    
    switch(cornerBlurp) {
      case 0:
      addForce(0.0f, 0.0f, vel, vel, c);
      break;
      case 1:
      addForce(1.0f, 0.0f, -vel, vel, c);
      break;
      case 2:
      addForce(0.0f, 1.0f, vel, -vel, c);
      break;
      case 3:
      addForce(1.0f, 1.0f, -vel, -vel, c);
      cornerBlurp = -1;
      break;
    }
    
    cornerBlurp++;
    
    lastCornerBlurp = millis();
  } 
}


public void doSideBlurps() {
 
 if(millis()-lastSideBlurp >= 500) {
   
   float vel = 0.5f;
   
   int c = generateColour(random(0,1.0f), random(0,1.0f));
    
    switch(sideBlurp) {
      case 0:
      addForce(0.5f, 0.0f, 0.0f, vel, c);
      break;
      case 1:
      addForce(1.0f, 0.5f, -vel, 0.0f, c);
      break;
      case 2:
      addForce(0.5f, 1.0f, 0.0f, -vel, c);
      break;
      case 3:
      addForce(0.0f, 0.5f, vel, 0.0f, c);
      sideBlurp = -1;
      break;
    }
   
   sideBlurp++;
   lastSideBlurp = millis();
   
 }
  
}


public void disturbance() {
  float vel = 0.3f;
  colorMode(HSB, 360, 1, 1);
  //color c = color(random(185, 250), 1.0, 1.0);
  int c = generateColour(0, 1);
  
  float xPos = 0.5f;//random(0.5-0.1, 0.5+0.1);
  float yPos = random(0.0f, 1.0f);
  float xVel = 0.3f;
  float yVel = 0.3f;
  
  if(disturbanceDir) {
    for(float i=0.1f; i<0.7f; i+=0.1f) {
      addForce( xPos, yPos, xVel+i, yVel+i, generateColour(xPos, yPos) );
    }
  } else {
    for(float i=0.1f; i<0.7f; i+=0.1f) {
      addForce( xPos, yPos, -(xVel+i), -(yVel+i), generateColour(xPos, yPos) );
    }
  }
  
  disturbanceDir = !disturbanceDir;
  
}


public void inkance() {
 
  float vel = 0.1f;
  int c = generateColour(0, 1);
  
  int d = 0;
  
  for(float j=0.1f; j<1.0f; j+=0.1f) {
    for(float i=0.1f; i<0.7f; i+=0.1f) {
      if(d%2 == 0) {
        addForce( 0.5f, j, vel+i, 0.0f, generateColour(0.5f, j) );  
      } else {
        addForce( 0.5f, j, -(vel+i), 0.0f, generateColour(0.5f, j) );
      }
    }
    d++;
  }
  
}


public void checkForFishies() {
 int count = 0;
 for(int i=0; i<numFish; i++) {
   float fishNormX = particleSystem.fishParticles[i].x*invWidth;
   if(fishNormX >= 0.5f-0.25f && fishNormX <= 0.5f+0.25f) {
     count++;
   }
 }
 if(count >= 0 && millis()-lastFishCountSend >= 1000) {
   if(connected) sendSerial(fishNumSerial, count);
   lastFishCountSend = millis();
 }
}


public void rotateFishes() {
 imageMode(CENTER);
  for (int i=0; i<5; i++) {
    pushMatrix();
    translate(width/2, height/2);
    translate(-width/2+particleSystem.fishParticles[i].x, -height/2+particleSystem.fishParticles[i].y);
    //rotate(radians(mouseY));
    //rotate(counter*TWO_PI/360);
    if(abs(particleSystem.fishParticles[i].vy) >= 1.0f || abs(particleSystem.fishParticles[i].vx) >= 1.0f) {
      fishAngle[i] = atan2(particleSystem.fishParticles[i].vy, particleSystem.fishParticles[i].vx);
    }
    rotate(fishAngle[i]);
    image(fishImg[i],0,0);
    translate(-fishImg[i].width/2, -fishImg[i].height/2);
    popMatrix();
  } 
}

// -- GUI -- //
ControlP5 cp5;
ControlP5 cp5_v2;
DropdownList d1;
boolean guiVisible = false;


public void initControlP5() {
  cp5 = new ControlP5(this);
  cp5_v2 = new ControlP5(this);
  cp5.setAutoDraw(false);
  cp5_v2.setAutoDraw(false);
  cp5.setMoveable(false);
  cp5_v2.setMoveable(false);
  
  loadGui();
  
  for (int i=0; i<Serial.list().length; i++) {
    if(DEBUG) println(Serial.list()[i]);
    d1.addItem("" + Serial.list()[i], i);
  }
  
  frame.setBackground(new java.awt.Color(0,0,0));
  
}


public void loadGui() {
 
  Group g0 = cp5.addGroup("g0")
                .setPosition(45,10)
                .setWidth(width-45*2)
                .setHeight(60)
                .setBackgroundHeight(60)
                .activateEvent(true)
                .setBackgroundColor(color(255,80))
                .setLabel("")
                .disableCollapse()
                .hideBar()
                ;
  
   d1 = cp5_v2.addDropdownList("serialports")
          .setPosition(360, 55)
          .setSize(200,150)
          .setBarHeight(30)
          .setCaptionLabel("Serial Ports")
          ;
          
  customize(d1);
  
  cp5.addButton("connect")
     .setValue(0)
     .setPosition(530,15)
     .setSize(100,30)
     .setGroup(g0)
     .setCaptionLabel("Connect!")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER);
     ;
  
}


public void customize(DropdownList ddl) {
  // a convenience function to customize a DropdownList
  ddl.setBackgroundColor(color(190));
  ddl.setItemHeight(30);
  //ddl.setBarHeight(15);
  ddl.getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER);
  //ddl.captionLabel().style().marginTop = 3;
  //ddl.captionLabel().style().marginLeft = 3;
  ddl.valueLabel().style().marginTop = 3;
}


public void controlEvent(ControlEvent theEvent) {
  // DropdownList is of type ControlGroup.
  // A controlEvent will be triggered from inside the ControlGroup class.
  // therefore you need to check the originator of the Event with
  // if (theEvent.isGroup())
  // to avoid an error message thrown by controlP5.

  if (theEvent.isGroup()) {
    // check if the Event was triggered from a ControlGroup
    //println("event from group : "+theEvent.getGroup().getValue()+" from "+theEvent.getGroup());
    
    if(theEvent.getGroup().getName().equals("serialports")) {
      port = (int)theEvent.getGroup().getValue();
      println("selected port: " + port);
    }
    
  } 
  else if (theEvent.isController()) {
    //println("event from controller : "+theEvent.getController().getValue()+" from "+theEvent.getController());
  }
}


public void connect(int theValue) {
  
  if(!connected) {
  
  if(port != 99) {
    println("connected");
    connected = true;
    arduino = new Serial(this, Serial.list()[port], 9600);
    cp5.getController("connect").setCaptionLabel("Disconnect");
  } else {
   println("need to select a port!"); 
  }
  
  } else {
    
    arduino.clear();
    arduino.stop();
    
    cp5.getController("connect").setCaptionLabel("Connect!");
    //port = 99;
    connected = false;
    
  }
  
}

  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "--full-screen", "--bgcolor=#666666", "--stop-color=#cccccc", "BotbaitFishing" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
